@@ -3,13 +3,15 @@ package FCGI::Buffer;
 use strict;
 use warnings;
 
+# FIXME: save_to treats ?arg1=a&arg2=b and ?arg2=b&arg1=a as different
+# FIXME: save_to treats /cgi-bin/foo.fcgi and /cgi-bin2/foo.fcgi as the same
+
 use Digest::MD5;
 use IO::String;
 use CGI::Info;
 use Carp;
 use HTTP::Date;
-
-# TODO: Encapsulate the data
+use DBI;
 
 =head1 NAME
 
@@ -17,11 +19,11 @@ FCGI::Buffer - Verify, Cache and Optimise FCGI Output
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =head1 SYNOPSIS
 
@@ -131,14 +133,14 @@ sub DESTROY {
 	my $self = shift;
 
 	select($self->{old_buf});
-	if(!defined($self->{buf})) {
+	if((!defined($self->{buf})) || (!defined($self->{buf}->getpos()))) {
 		# Unlikely
 		if($self->{'logger'}) {
 			$self->{'logger'}->info('Nothing to send');
 		}
 		return;
 	}
-	my $pos = $self->{buf}->getpos;
+	my $pos = $self->{buf}->getpos();
 	$self->{buf}->setpos(0);
 	my $buf;
 	read($self->{buf}, $buf, $pos);
@@ -226,6 +228,16 @@ sub DESTROY {
 
 				$self->{body} =~ s/<a\s+?href="$protocol:\/\/$href"/<a href="\/"/gim;
 				$self->{body} =~ s/<a\s+?href="$protocol:\/\/$href/<a href="/gim;
+				$self->{body} =~ s/<a\s+?href="$protocol:\/\//<a href="\/\//gim;
+
+				# If we're in "/cgi-bin/foo.cgi?arg1=a" replace
+				# "/cgi-bin/foo.cgi?arg2=b" with "?arg2=b"
+
+				if(my $script_name = $ENV{'SCRIPT_NAME'}) {
+					if($script_name =~ /^\//) {
+						$self->{body} =~ s/<a\s+?href="$script_name(\?.+)?"/<a href="$1"/gim;
+					}
+				}
 
 				# TODO use URI->path_segments to change links in
 				# /aa/bb/cc/dd.htm which point to /aa/bb/ff.htm to
@@ -276,6 +288,7 @@ sub DESTROY {
 				}
 				my $lint = HTML::Lint->new();
 				$lint->parse($self->{body});
+				$lint->eof();
 
 				if($lint->errors) {
 					$headers = 'Status: 500 Internal Server Error';
@@ -316,6 +329,22 @@ sub DESTROY {
 		}
 	}
 
+	my $dbh;
+	if(my $save_to = $self->{save_to}) {
+		my $sqlite_file = $save_to->{directory} . '/fcgi.buffer.sql';
+		if(!-r $sqlite_file) {
+			if(!-d $save_to->{directory}) {
+				mkdir $save_to->{directory};
+			}
+			$dbh = DBI->connect("dbi:SQLite:dbname=$sqlite_file", undef, undef);
+			if($self->{save_to}->{create_table}) {
+				$dbh->prepare('CREATE TABLE fcgi_buffer(key char PRIMARY KEY, language char, browser_type char, path char UNIQUE NOT NULL, uri char NOT NULL, creation timestamp NOT NULL)')->execute();
+			}
+		} else {
+			$dbh = DBI->connect("dbi:SQLite:dbname=$sqlite_file", undef, undef);
+		}
+	}
+
 	my $encoding = $self->_should_gzip();
 	my $unzipped_body = $self->{body};
 
@@ -333,6 +362,7 @@ sub DESTROY {
 					$self->{body} = substr($self->{body}, 0, $2);
 				}
 				$unzipped_body = $self->{body};
+				$self->{'status'} = 206;
 			}
 		}
 		$self->_compress({ encoding => $encoding });
@@ -380,6 +410,12 @@ sub DESTROY {
 			}
 			if($self->{send_body} && ($self->{status} == 200)) {
 				$self->{body} = $cache_hash->{'body'};
+				if($dbh) {
+					my $changes = $self->_save_to($self->{body}, $dbh);
+					if($changes && (my $ttl = $self->{save_to}->{ttl})) {
+						push @{$self->{o}}, 'Expires: ' . HTTP::Date::time2str(time + $ttl);
+					}
+				}
 				if(!defined($self->{body})) {
 					# Panic
 					$headers = 'Status: 500 Internal Server Error';
@@ -388,7 +424,27 @@ sub DESTROY {
 					foreach my $k (keys %{$cache_hash}) {
 						$self->{body} .= "\t$k\n";
 					}
+
+					if($dbh) {
+						my $query = "SELECT DISTINCT path FROM fcgi_buffer WHERE key = '$key'";
+						my $sth = $dbh->prepare($query);
+						if($self->{logger}) {
+							$self->{logger}->debug($query);
+						}
+						if($sth->execute() && (my $href = $sth->fetchrow_hashref())) {
+							if(my $path = $href->{'path'}) {
+								unlink($path);
+							}
+						}
+						$query = "DELETE FROM fcgi_buffer WHERE key = '$key'";
+						$dbh->prepare($query)->execute();
+						if($self->{logger}) {
+							$self->{logger}->debug($query);
+						}
+					}
+
 					$self->{cache}->remove($key);
+
 					if($self->{logger}) {
 						$self->{logger}->error("Can't retrieve body for key $key");
 					} else {
@@ -452,6 +508,8 @@ sub DESTROY {
 		} else {
 			# Not in the server side cache
 			if($self->{status} == 200) {
+				my $changes = $self->_save_to($unzipped_body, $dbh);
+
 				unless($self->{cache_age}) {
 					# It would be great if CHI::set()
 					# allowed the time to be 'lru' for least
@@ -459,6 +517,9 @@ sub DESTROY {
 					$self->{cache_age} = '10 minutes';
 				}
 				$cache_hash->{'body'} = $unzipped_body;
+				if($changes && $encoding) {
+					$self->_compress({ encoding => $encoding });
+				}
 				if($self->{o} && scalar(@{$self->{o}})) {
 					# Remember, we're storing the UNzipped
 					# version in the cache
@@ -492,6 +553,75 @@ sub DESTROY {
 					$self->{logger}->debug("Store $key in the cache, age = ", $self->{cache_age}, ' ', length($cache_hash->{'body'}), ' bytes');
 				}
 				$self->{cache}->set($key, Storable::freeze($cache_hash), $self->{cache_age});
+
+				# Create a static page with the information and link to that in the output
+				# HTML
+				if($dbh && $self->{info} && $self->{save_to} && (my $request_uri = $ENV{'REQUEST_URI'})) {
+					my $query = "SELECT DISTINCT creation FROM fcgi_buffer WHERE key = ?";
+					if($self->{logger}) {
+						$self->{logger}->debug("$query: $key");
+					}
+					my $sth = $dbh->prepare($query);
+					$sth->execute($key);
+					if(my $href = $sth->fetchrow_hashref()) {
+						if(my $ttl = $self->{save_to}->{ttl}) {
+							push @{$self->{o}}, 'Expires: ' .
+								HTTP::Date::time2str($href->{'creation'} + $ttl);
+						}
+					} else {
+						my $dir = $self->{save_to}->{directory};
+						my $browser_type = $self->{info}->browser_type();
+						my $language = $self->{lingua}->language();
+						my $bdir = "$dir/$browser_type";
+						if($bdir =~ /^(.+)$/) {
+							$bdir = $1; # Untaint
+						}
+						my $ldir = "$bdir/$language";
+						my $script_name = $self->{info}->script_name();
+						my $sdir = "$ldir/$script_name";
+						if(!-d $bdir) {
+							mkdir $bdir;
+							mkdir $ldir;
+							mkdir $sdir;
+						} elsif(!-d $ldir) {
+							mkdir $ldir;
+							mkdir $sdir;
+						} elsif(!-d $sdir) {
+							mkdir $sdir;
+						}
+						my $path = "$sdir/" . $self->{info}->as_string() . '.html';
+						if($path =~ /^(.+)$/) {
+							$path = $1; # Untaint
+							$path =~ tr/[\|;]/_/;
+						}
+						if(open(my $fout, '>', $path)) {
+							my $u = $request_uri;
+							$u =~ s/\?/\\?/g;
+							my $copy = $unzipped_body;
+							my $changes = ($copy =~ s/<a\s+href="$u"/<a href="$path"/gi);
+
+							# handle <a href="?arg3=4">Call self with different args</a>
+							$script_name = $ENV{'SCRIPT_NAME'};
+							$copy =~ s/<a\s+href="(\?.+?)"/<a href="$script_name$1"/gi;
+
+							print $fout $copy;
+							close $fout;
+							# Do INSERT OR REPLACE in case another program has
+							# got in first,
+							$query = "INSERT OR REPLACE INTO fcgi_buffer(key, language, browser_type, path, uri, creation) VALUES('$key', '$language', '$browser_type', '$path', '$request_uri', strftime('\%s','now'))";
+							if($self->{logger}) {
+								$self->{logger}->debug($query);
+							}
+							$dbh->prepare($query)->execute();
+
+							if($changes && (my $ttl = $self->{save_to}->{ttl})) {
+								push @{$self->{o}}, 'Expires: ' . HTTP::Date::time2str(time + $ttl);
+							}
+						} elsif($self->{logger}) {
+							$self->{logger}->warn("Can't create $path");
+						}
+					}
+				}
 				if($self->{generate_last_modified}) {
 					$self->{cobject} = $self->{cache}->get_object($key);
 					if(defined($self->{cobject})) {
@@ -525,6 +655,9 @@ sub DESTROY {
 	} elsif($self->{info}) {
 		my $host_name = $self->{info}->host_name();
 		push @{$self->{o}}, ("X-Cache: MISS from $host_name", "X-Cache-Lookup: MISS from $host_name");
+		if($self->_save_to($unzipped_body, $dbh) && $encoding) {
+			$self->_compress({ encoding => $encoding });
+		}
 	} else {
 		push @{$self->{o}}, ('X-Cache: MISS', 'X-Cache-Lookup: MISS');
 	}
@@ -555,10 +688,16 @@ sub DESTROY {
 		# FCGI::Buffer
 		unshift @{$self->{o}}, split(/\r\n/, $headers);
 		if($self->{body} && $self->{send_body}) {
-			unless(grep(/^Content-Length: /, $self->{o})) {
+			unless(grep(/^Content-Length: \d/, @{$self->{o}})) {
 				push @{$self->{o}}, "Content-Length: $body_length";
 			}
 		}
+		unless(grep(/^Status: \d/, @{$self->{o}})) {
+			require HTTP::Status;
+			HTTP::Status->import();
+
+			push @{$self->{o}}, 'Status: ' . $self->{status} . ' ' . HTTP::Status::status_message($self->{status});
+		}
 	} else {
 		push @{$self->{o}}, "X-FCGI-Buffer-$VERSION: No headers";
 	}
@@ -583,7 +722,7 @@ sub DESTROY {
 					$self->{logger}->trace($call_details[1], ':', $call_details[2], ' in function ', $call_details[3]);
 				}
 			}
-			CORE::warn(@_);     # call the builtin warn as usual
+			CORE::warn(@_);	# call the builtin warn as usual
 		};
 
 		if(scalar @{$self->{o}}) {
@@ -676,7 +815,7 @@ sub _optimise_content {
 	# $self->{body} =~ s/\<div\>\s+/\<div\>/gis;	# Remove spaces after <div>
 	$self->{body} =~ s/(<div>\s+|\s+<div>)/<div>/gis;
 	$self->{body} =~ s/\s+<\/div\>/\<\/div\>/gis;	# Remove spaces before </div>
-	$self->{body} =~ s/\s+\<p\>|\<p\>\s+/\<p\>/im;  # TODO <p class=
+	$self->{body} =~ s/\s+\<p\>|\<p\>\s+/\<p\>/im;	# TODO <p class=
 	$self->{body} =~ s/\s+\<\/p\>|\<\/p\>\s+/\<\/p\>/gis;
 	$self->{body} =~ s/<html>\s+<head>/<html><head>/is;
 	$self->{body} =~ s/\s*<\/head>\s+<body>\s*/<\/head><body>/is;
@@ -693,7 +832,8 @@ sub _optimise_content {
 	$self->{body} =~ s/\<\/td\>\s*\<td\>/\<\/td\>\<td\>/gis;
 	$self->{body} =~ s/\<\/tr\>\s\<\/table\>/\<\/tr\>\<\/table\>/gi;
 	$self->{body} =~ s/\<br\s?\/?\>\s?\<p\>/\<p\>/gi;
-	$self->{body} =~ s/\<br\>\s/\<br\>/gi;
+	$self->{body} =~ s/\<br\>\s+/\<br\>/gi;
+	$self->{body} =~ s/\s+\<br\>/\<br\>/gi;
 	$self->{body} =~ s/\<br\s?\/\>\s/\<br \/\>/gi;
 	$self->{body} =~ s/[ \t]+/ /gs;	# Remove duplicate space, don't use \s+ it breaks JavaScript
 	$self->{body} =~ s/\s\<p\>/\<p\>/gi;
@@ -724,9 +864,11 @@ sub _generate_key {
 		$self->{info} = CGI::Info->new({ cache => $self->{cache} });
 	}
 
-	# TODO: Use CGI::Lingua so that different languages are stored
-	#	in different caches
 	my $key = $self->{info}->browser_type() . '::' . $self->{info}->domain_name() . '::' . $self->{info}->script_name() . '::' . $self->{info}->as_string();
+
+	if($self->{lingua}) {
+		$key .= '::' . $self->{lingua}->language();
+	}
 	if($ENV{'HTTP_COOKIE'}) {
 		# Different states of the client are stored in different caches
 		# Don't put different Google Analytics in different caches, and anyway they
@@ -776,8 +918,8 @@ Set various options and override default values.
     # optimise_content and lint_content are OFF.  Set optimise_content to 2 to
     # do aggressive JavaScript optimisations which may fail.
     use FCGI::Buffer;
-    my $buffer = FCGI::Buffer->new();
-    $buffer->init({
+
+    my $buffer = FCGI::Buffer->new()->init({
 	generate_etag => 1,	# make good use of client's cache
 	generate_last_modified => 1,	# more use of client's cache
 	compress_content => 1,	# if gzip the output
@@ -787,8 +929,11 @@ Set various options and override default values.
 	cache_age => '10 minutes',	# how long to store responses in the cache
 	logger => $self->{logger},
 	lint_content => 0,	# Pass through HTML::Lint
-	generate_304 => 1,	# Generate 304: Not modified
-    );
+	generate_304 => 1,	# When appropriate, generate 304: Not modified
+	save_to => { directory => '/var/www/htdocs/save_to', ttl => 600, create_table => 1 },
+	info => CGI::Info->new(),
+	lingua => CGI::Lingua->new(),
+    });
 
 If no cache_key is given, one will be generated which may not be unique.
 The cache_key should be a unique value dependent upon the values set by the
@@ -802,6 +947,20 @@ Items stay in the server-side cache by default for 10 minutes.
 This can be overridden by the cache_control HTTP header in the request, and
 the default can be changed by the cache_age argument to init().
 
+Save_to is feature which stores output of dynamic pages to your
+htdocs tree and replaces future links that point to that page with static links
+to avoid going through CGI at all.
+Ttl is set to the number of seconds that the static pages are deemed to
+be live for, the default is 10 minutes.
+If set to 0, the page is live forever.
+To enable save_to, a info and lingua arguments must also be given.
+It works best when cache is also given.
+Only use where output is guaranteed to be the same with a given set of arguments
+(the same criteria for enabling generate_304).
+
+Info is an optional argument to give information about the FCGI environment, e.g.
+a L<CGI::Info> object.
+
 Logger will be an object that understands debug() such as an L<Log::Log4perl>
 object.
 
@@ -847,6 +1006,16 @@ sub init {
 	if(defined($params{logger})) {
 		$self->{logger} = $params{logger};
 	}
+	if(defined($params{lingua})) {
+		$self->{lingua} = $params{lingua};
+	}
+	# Don't forget to handle where lingua could have been set in a previous init() call
+	if(defined($params{save_to}) && $self->{lingua} && $self->can_cache()) {
+		$self->{save_to} = $params{save_to};
+		if(!exists($params{save_to})) {
+			$self->{save_to} = 600;
+		}
+	}
 	if(defined($params{generate_304})) {
 		$self->{generate_304} = $params{generate_304};
 	}
@@ -897,13 +1066,15 @@ sub init {
 			$self->{cache_key} = $params{cache_key};
 		}
 	}
+
+	return $self;
 }
 
 sub import {
 	# my $class = shift;
 	shift;
 
-	return unless @_;
+	return unless scalar(@_);
 
 	init(@_);
 }
@@ -1081,25 +1252,17 @@ sub _my_age {
 sub _should_gzip {
 	my $self = shift;
 
-	if(defined($self->{send_body}) && ($self->{send_body} == 0)) {
-		return '';
-	}
-
 	if($self->{compress_content} && ($ENV{'HTTP_ACCEPT_ENCODING'} || $ENV{'HTTP_TE'})) {
+		if(defined($self->{content_type})) {
+			my @content_type = @{$self->{content_type}};
+			if($content_type[0] ne 'text') {
+				return '';
+			}
+		}
 		my $accept = lc($ENV{'HTTP_ACCEPT_ENCODING'} ? $ENV{'HTTP_ACCEPT_ENCODING'} : $ENV{'HTTP_TE'});
-		foreach my $encoding ('x-gzip', 'gzip', 'br') {
-			$_ = $accept;
-			if(defined($self->{content_type})) {
-				my @content_type = @{$self->{content_type}};
-				if($content_type[0]) {
-					if (m/$encoding/i && (lc($content_type[0]) eq 'text')) {
-						return $encoding;
-					}
-				} else {
-					if (m/$encoding/i) {
-						return $encoding;
-					}
-				}
+		foreach my $method(split(/,\s?/, $accept)) {
+			if(($method eq 'gzip') || ($method eq 'x-gzip') || ($method eq 'br')) {
+				return $method;
 			}
 		}
 	}
@@ -1145,23 +1308,30 @@ sub _compress()
 		my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
 		if(length($nbody) < length($self->{body})) {
 			$self->{body} = $nbody;
-			push @{$self->{o}}, "Content-Encoding: $encoding";
-			push @{$self->{o}}, "Vary: Accept-Encoding";
+			unless(grep(/^Content-Encoding: gzip/, @{$self->{o}})) {
+				push @{$self->{o}}, 'Content-Encoding: gzip';
+			}
+			unless(grep(/^Vary: Accept-Encoding/, @{$self->{o}})) {
+				push @{$self->{o}}, 'Vary: Accept-Encoding';
+			}
 		}
 	} elsif($encoding eq 'br') {
 		require IO::Compress::Brotli;
 		IO::Compress::Brotli->import();
 
-		# Avoid 'Wide character in memGzip'
 		unless($self->{_encode_loaded}) {
 			require Encode;
 			$self->{_encode_loaded} = 1;
 		}
-		my $nbody = bro(Encode::encode_utf8($self->{body}));
+		my $nbody = IO::Compress::Brotli::bro(Encode::encode_utf8($self->{body}));
 		if(length($nbody) < length($self->{body})) {
 			$self->{body} = $nbody;
-			push @{$self->{o}}, "Content-Encoding: $encoding";
-			push @{$self->{o}}, "Vary: Accept-Encoding";
+			unless(grep(/^Content-Encoding: br/, @{$self->{o}})) {
+				push @{$self->{o}}, 'Content-Encoding: br';
+			}
+			unless(grep(/^Vary: Accept-Encoding/, @{$self->{o}})) {
+				push @{$self->{o}}, 'Vary: Accept-Encoding';
+			}
 		}
 	}
 }
@@ -1173,7 +1343,7 @@ sub _check_if_none_match {
 		$self->{logger}->debug("Compare $ENV{HTTP_IF_NONE_MATCH} with $self->{etag}");
 	}
 	if($ENV{'HTTP_IF_NONE_MATCH'} eq $self->{etag}) {
-		push @{$self->{o}}, "Status: 304 Not Modified";
+		push @{$self->{o}}, 'Status: 304 Not Modified';
 		$self->{send_body} = 0;
 		$self->{status} = 304;
 		if($self->{logger}) {
@@ -1198,6 +1368,139 @@ sub _check_if_none_match {
 	return 0;
 }
 
+# replace dynamic links with static links
+sub _save_to {
+	my ($self, $unzipped_body, $dbh) = @_;
+
+	return 0 unless($dbh && $self->{info} && (my $request_uri = $ENV{'REQUEST_URI'}));
+
+	my $query;
+	my $copy = $unzipped_body;
+	my $changes = 0;
+	my $creation;
+	my %seen_links;
+	while($unzipped_body =~ /<a\shref="(.+?)"/gis) {
+		my $link = $1;
+		next if($seen_links{$link});	# Already updated in the copy
+		$seen_links{$link} = 1;
+		$link =~ tr/[\|;]/_/;
+
+		my $search_uri = $link;
+		if($search_uri =~ /^\?/) {
+			# CGI script has links to itself
+			# $search_uri = "${request_uri}${link}";
+			my $r = $request_uri;
+			$r =~ s/\?.*$//;
+			$search_uri = "${r}$link";
+		} else {
+			next if($link =~ /^https?:\/\//);	# FIXME: skips full URLs to ourself
+								#	Though optimise_content fixes that
+			next if($link =~ /.html?$/);
+			next if($link =~ /.jpg$/);
+			next if($link =~ /.gif$/);
+		}
+		if($self->{save_to}->{ttl}) {
+			$query = "SELECT DISTINCT path, creation FROM fcgi_buffer WHERE uri = ? AND language = ? AND browser_type = ? AND creation >= strftime('\%s','now') - " . $self->{save_to}->{ttl};
+		} else {
+			$query = "SELECT DISTINCT path, creation FROM fcgi_buffer WHERE uri = ? AND language = ? AND browser_type = ?";
+		}
+		if($self->{logger}) {
+			$self->{logger}->debug("$query: $search_uri");
+		}
+		my $sth = $dbh->prepare($query);
+		if(!defined($sth)) {
+			if($self->{logger}) {
+				$self->{logger}->warn("failed to prepare '$query'");
+			}
+		} else {
+			$sth->execute($search_uri, $self->{lingua}->language(), $self->{info}->browser_type());
+			if(my $href = $sth->fetchrow_hashref()) {
+				if(my $path = $href->{'path'}) {
+					if(-r $path) {
+						$link =~ s/\?/\\?/g;
+						my $rootdir = $self->{info}->rootdir();
+						$path =~ s/^$rootdir//;
+						$changes += ($copy =~ s/<a\s+href="$link">/<a href="$path">/gis);
+						# Find the first link that will expire and use that
+						if((!defined($creation)) || ($href->{'creation'} < $creation)) {
+							$creation = $href->{'creation'};
+						}
+					} else {
+						$query = "DELETE FROM fcgi_buffer WHERE path = ?";
+						$dbh->prepare($query)->execute($path);
+						if($self->{logger}) {
+							$self->{logger}->warn("Remove entry for non-existant file $path");
+						}
+					}
+				}
+			}
+		}
+	};
+	my $expiration = 0;
+	if(defined($creation) && (my $ttl = $self->{save_to}->{ttl})) {
+		$expiration = $creation + $ttl;
+	}
+	if($changes && (($expiration == 0) || ($expiration >= time))) {
+		if($self->{logger}) {
+			# $self->{logger}->debug("$changes links now point to static pages");
+			$self->{logger}->info("$changes links now point to static pages");
+		}
+		$unzipped_body = $copy;
+		$self->{'body'} = $unzipped_body;
+		if(my $ttl = $self->{save_to}->{ttl}) {
+			push @{$self->{o}}, 'Expires: ' . HTTP::Date::time2str($creation + $ttl);
+		}
+	} elsif($expiration && ($expiration < time)) {
+		# Delete the save_to files
+		if($self->{save_to}->{ttl}) {
+			$query = "SELECT path FROM fcgi_buffer WHERE creation < strftime('\%s','now') - " . $self->{save_to}->{ttl};
+		} else {
+			$query = 'SELECT path FROM fcgi_buffer';	# Hmm, I suspect this is overkill
+		}
+		my $sth = $dbh->prepare($query);
+		$sth->execute();
+		while(my $href = $sth->fetchrow_hashref()) {
+			if(my $path = $href->{'path'}) {
+				if($self->{logger}) {
+					$self->{logger}->debug("unlink $path");
+				}
+				unlink $path;
+			}
+		}
+		if($self->{save_to}->{ttl}) {
+			$query = "DELETE FROM fcgi_buffer WHERE creation < strftime('\%s','now') - " . $self->{save_to}->{ttl};
+		} else {
+			$query = 'DELETE FROM fcgi_buffer';	# Hmm, I suspect this is overkill
+		}
+		if($self->{logger}) {
+			$self->{logger}->debug($query);
+		}
+		$dbh->prepare($query)->execute();
+	# } else {
+		# Old code
+		# if($self->{save_to}->{ttl}) {
+			# $query = "SELECT DISTINCT path, creation FROM fcgi_buffer WHERE key = '$key' AND creation >= strftime('\%s','now') - " . $self->{save_to}->{ttl};
+		# } else {
+			# $query = "SELECT DISTINCT path, creation FROM fcgi_buffer WHERE key = '$key'";
+		# }
+		# my $sth = $dbh->prepare($query);
+		# $sth->execute();
+		# my $href = $sth->fetchrow_hashref();
+		# if(my $path = $href->{'path'}) {
+			# # FIXME: don't do this if we've passed the TTL, and if we are clean
+			# #	up the database and remove the static page
+			# $request_uri =~ s/\?/\\?/g;
+			# if(($unzipped_body =~ s/<a href="$request_uri"/<a href="$path"/gi) > 0) {
+				# $self->{'body'} = $unzipped_body;
+				# if(my $ttl = $self->{save_to}->{ttl}) {
+					# push @{$self->{o}}, 'Expires: ' . HTTP::Date::time2str($href->{creation} + $ttl);
+				# }
+			# }
+		# }
+	}
+	return $changes;
+}
+
 =head1 AUTHOR
 
 Nigel Horne, C<< <njh at bandsman.co.uk> >>
@@ -1240,6 +1543,10 @@ L<Log::Log4Perl>, if you're using that, so that any messages it
 produces are printed after the HTTP headers have been sent by
 FCGI::Buffer;
 
+Save_to doesn't understand links in JavaScript, which means that if you use self-calling
+CGIs which are loaded as a static page they may point to the wrong place.
+The workaround is to avoid self-calling CGIs in JavaScript
+
 Please report any bugs or feature requests to C<bug-fcgi-buffer at rt.cpan.org>,
 or through the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=FCGI-Buffer>.
 I will be notified, and then you'll automatically be notified of progress on
@@ -1279,7 +1586,7 @@ L<http://search.cpan.org/dist/FCGI-Buffer/>
 
 =head1 ACKNOWLEDGEMENTS
 
-The inspiration and code for some if this is cgi_buffer by Mark
+The inspiration and code for some of this is cgi_buffer by Mark
 Nottingham: http://www.mnot.net/cgi_buffer.
 
 =head1 LICENSE AND COPYRIGHT
@@ -1293,7 +1600,7 @@ The licence for cgi_buffer is:
 
     This software is provided 'as is' without warranty of any kind."
 
-The rest of the program is Copyright 2015-2016 Nigel Horne,
+The rest of the program is Copyright 2015-2017 Nigel Horne,
 and is released under the following licence: GPL
 
 =cut
